@@ -29,14 +29,7 @@ public:
   static bool isProvidesLine(const std::string& line, std::string& pkgName);
   static std::string getPkgFileName(const std::string& section);
   static void extractProvidesReferences(const std::string& section, StringSet& refs);
-
-private:
-  static std::string encodeMultiline(const std::string& s);
-  static std::string encodeChangeLogEntry(const ChangeLogEntry& entry);
-  static std::string saveNamedPkgRel(const NamedPkgRel& r);
-  static std::string saveFileName(const std::string& fileName);
-  static bool fileFromDirs(const std::string& fileName, const StringVector& dirs);
-  static std::string extractPkgRelName(const std::string& line);
+  static bool parsePkgFileSection(const StringVector& sect, PkgFile& pkgFile, size_t& invalidLineNum, std::string& invalidLineValue);
 }; //class PkgSection;
 
 #endif //DEEPSOLVER_PKG_SECTION_H;
