@@ -164,7 +164,7 @@ void initCliParser()
 
 void printLogo()
 {
-  std::cout << "ds-repo: the utility to create Deepsolver repository index" << std::endl;
+  std::cout << "ds-repo: The utility to create Deepsolver repository index" << std::endl;
     std::cout << "Version: " << PACKAGE_VERSION << std::endl;
   std::cout << std::endl;
 }
@@ -195,6 +195,7 @@ void parseCmdLine(int argc, char* argv[])
 	  exit(EXIT_FAILURE);
 	case CliParserException::MissedArgument:
 	  std::cerr << PREFIX << "Command line argument \'" << e.getArg() << "\' requires additional parameter" << std::endl;exit(EXIT_FAILURE);
+	  exit(EXIT_FAILURE);
 	default:
 	  assert(0);
 	} //switch();
